@@ -17,8 +17,10 @@
 
 use pyo3::prelude::*;
 
+mod data_file;
 mod datafusion_table_provider;
 mod error;
+mod manifest;
 mod runtime;
 mod transform;
 
@@ -26,5 +28,6 @@ mod transform;
 fn pyiceberg_core_rust(py: Python<'_>, m: &Bound<'_, PyModule>) -> PyResult<()> {
     datafusion_table_provider::register_module(py, m)?;
     transform::register_module(py, m)?;
+    manifest::register_module(py, m)?;
     Ok(())
 }
