@@ -1996,7 +1996,7 @@ mod test {
             upper_bounds: Default::default(),
             key_metadata: None,
             split_offsets: vec![],
-            equality_ids: vec![],
+            equality_ids: None,
             sort_order_id: None,
             partition_spec_id: 0,
             first_row_id: None,
@@ -2022,7 +2022,7 @@ mod test {
             upper_bounds: Default::default(),
             key_metadata: None,
             split_offsets: vec![],
-            equality_ids: vec![],
+            equality_ids: None,
             sort_order_id: None,
             partition_spec_id: 0,
             first_row_id: None,
@@ -2084,7 +2084,7 @@ mod test {
             column_sizes: Default::default(),
             key_metadata: None,
             split_offsets: vec![],
-            equality_ids: vec![],
+            equality_ids: None,
             sort_order_id: None,
             partition_spec_id: 0,
             first_row_id: None,
@@ -2115,7 +2115,7 @@ mod test {
             column_sizes: Default::default(),
             key_metadata: None,
             split_offsets: vec![],
-            equality_ids: vec![],
+            equality_ids: None,
             sort_order_id: None,
             partition_spec_id: 0,
             first_row_id: None,
@@ -2147,7 +2147,7 @@ mod test {
             column_sizes: Default::default(),
             key_metadata: None,
             split_offsets: vec![],
-            equality_ids: vec![],
+            equality_ids: None,
             sort_order_id: None,
             partition_spec_id: 0,
             first_row_id: None,
@@ -2179,7 +2179,7 @@ mod test {
             column_sizes: Default::default(),
             key_metadata: None,
             split_offsets: vec![],
-            equality_ids: vec![],
+            equality_ids: None,
             sort_order_id: None,
             partition_spec_id: 0,
             first_row_id: None,
