@@ -579,7 +579,7 @@ mod test {
             column_sizes: Default::default(),
             key_metadata: None,
             split_offsets: vec![],
-            equality_ids: vec![],
+            equality_ids: None,
             sort_order_id: None,
             partition_spec_id: 0,
             first_row_id: None,
@@ -605,7 +605,7 @@ mod test {
             upper_bounds: Default::default(),
             key_metadata: None,
             split_offsets: vec![],
-            equality_ids: vec![],
+            equality_ids: None,
             sort_order_id: None,
             partition_spec_id: 0,
             first_row_id: None,
@@ -631,7 +631,7 @@ mod test {
             column_sizes: Default::default(),
             key_metadata: None,
             split_offsets: vec![],
-            equality_ids: vec![],
+            equality_ids: None,
             sort_order_id: None,
             partition_spec_id: 0,
             first_row_id: None,
@@ -658,7 +658,7 @@ mod test {
             column_sizes: Default::default(),
             key_metadata: None,
             split_offsets: vec![],
-            equality_ids: vec![],
+            equality_ids: None,
             sort_order_id: None,
             partition_spec_id: 0,
             first_row_id: None,
