@@ -15,6 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use std::any::{Any, TypeId};
 use std::collections::HashMap;
 use std::ops::Range;
 use std::sync::Arc;
@@ -167,6 +168,31 @@ impl FileIO {
     }
 }
 
+/// Container for storing type-safe extensions used to configure underlying FileIO behavior.
+#[derive(Clone, Debug, Default)]
+pub struct Extensions(HashMap<TypeId, Arc<dyn Any + Send + Sync>>);
+
+impl Extensions {
+    /// Add an extension.
+    pub fn add<T: Any + Send + Sync>(&mut self, ext: T) {
+        self.0.insert(TypeId::of::<T>(), Arc::new(ext));
+    }
+
+    /// Extends the current set of extensions with another set of extensions.
+    pub fn extend(&mut self, extensions: Extensions) {
+        self.0.extend(extensions.0);
+    }
+
+    /// Fetch an extension.
+    pub fn get<T>(&self) -> Option<Arc<T>>
+    where T: 'static + Send + Sync + Clone {
+        let type_id = TypeId::of::<T>();
+        self.0
+            .get(&type_id)
+            .and_then(|arc_any| Arc::clone(arc_any).downcast::<T>().ok())
+    }
+}
+
 /// Builder for [`FileIO`].
 #[derive(Clone, Debug)]
 pub struct FileIOBuilder {
@@ -176,6 +202,8 @@ pub struct FileIOBuilder {
     scheme_str: Option<String>,
     /// Arguments for operator.
     props: HashMap<String, String>,
+    /// Optional extensions to configure the underlying FileIO behavior.
+    extensions: Extensions,
 }
 
 impl FileIOBuilder {
@@ -185,6 +213,7 @@ impl FileIOBuilder {
         Self {
             scheme_str: Some(scheme_str.to_string()),
             props: HashMap::default(),
+            extensions: Extensions::default(),
         }
     }
 
@@ -193,14 +222,19 @@ impl FileIOBuilder {
         Self {
             scheme_str: None,
             props: HashMap::default(),
+            extensions: Extensions::default(),
         }
     }
 
     /// Fetch the scheme string.
     ///
     /// The scheme_str will be empty if it's None.
-    pub fn into_parts(self) -> (String, HashMap<String, String>) {
-        (self.scheme_str.unwrap_or_default(), self.props)
+    pub fn into_parts(self) -> (String, HashMap<String, String>, Extensions) {
+        (
+            self.scheme_str.unwrap_or_default(),
+            self.props,
+            self.extensions,
+        )
     }
 
     /// Add argument for operator.
@@ -219,6 +253,24 @@ impl FileIOBuilder {
         self
     }
 
+    /// Add an extension to the file IO builder.
+    pub fn with_extension<T: Any + Send + Sync>(mut self, ext: T) -> Self {
+        self.extensions.add(ext);
+        self
+    }
+
+    /// Adds multiple extensions to the file IO builder.
+    pub fn with_extensions(mut self, extensions: Extensions) -> Self {
+        self.extensions.extend(extensions);
+        self
+    }
+
+    /// Fetch an extension from the file IO builder.
+    pub fn extension<T>(&self) -> Option<Arc<T>>
+    where T: 'static + Send + Sync + Clone {
+        self.extensions.get::<T>()
+    }
+
     /// Builds [`FileIO`].
     pub fn build(self) -> Result<FileIO> {
         let storage = Storage::build(self.clone())?;
@@ -338,6 +390,17 @@ impl FileWrite for opendal::Writer {
     }
 }
 
+#[async_trait::async_trait]
+impl FileWrite for Box<dyn FileWrite> {
+    async fn write(&mut self, bs: Bytes) -> crate::Result<()> {
+        self.as_mut().write(bs).await
+    }
+
+    async fn close(&mut self) -> crate::Result<()> {
+        self.as_mut().close().await
+    }
+}
+
 /// Output file is used for writing to files..
 #[derive(Debug)]
 pub struct OutputFile {
