@@ -266,7 +266,7 @@ mod tests {
             let mut writer = ManifestWriterBuilder::new(
                 self.next_manifest_file(),
                 Some(current_snapshot.snapshot_id()),
-                vec![],
+                None,
                 current_schema.clone(),
                 current_partition_spec.as_ref().clone(),
             )
