@@ -18,10 +18,11 @@
 
 use std::collections::HashMap;
 
-use opendal::services::GcsConfig;
 use opendal::Operator;
+use opendal::services::GcsConfig;
 use url::Url;
 
+use crate::io::is_truthy;
 use crate::{Error, ErrorKind, Result};
 
 // Reference: https://github.com/apache/iceberg/blob/main/gcp/src/main/java/org/apache/iceberg/gcp/GCPProperties.java
@@ -41,6 +42,13 @@ pub const GCS_CREDENTIALS_JSON: &str = "gcs.credentials-json";
 /// Google Cloud Storage token
 pub const GCS_TOKEN: &str = "gcs.oauth2.token";
 
+/// Option to skip signing requests (e.g. for public buckets/folders).
+pub const GCS_ALLOW_ANONYMOUS: &str = "gcs.allow-anonymous";
+/// Option to skip loading the credential from GCE metadata server (typically used in conjunction with `GCS_ALLOW_ANONYMOUS`).
+pub const GCS_DISABLE_VM_METADATA: &str = "gcs.disable-vm-metadata";
+/// Option to skip loading configuration from config file and the env.
+pub const GCS_DISABLE_CONFIG_LOAD: &str = "gcs.disable-config-load";
+
 /// Parse iceberg properties to [`GcsConfig`].
 pub(crate) fn gcs_config_parse(mut m: HashMap<String, String>) -> Result<GcsConfig> {
     let mut cfg = GcsConfig::default();
@@ -63,6 +71,22 @@ pub(crate) fn gcs_config_parse(mut m: HashMap<String, String>) -> Result<GcsConf
         cfg.disable_config_load = true;
     }
 
+    if let Some(allow_anonymous) = m.remove(GCS_ALLOW_ANONYMOUS) {
+        if is_truthy(allow_anonymous.to_lowercase().as_str()) {
+            cfg.allow_anonymous = true;
+        }
+    }
+    if let Some(disable_ec2_metadata) = m.remove(GCS_DISABLE_VM_METADATA) {
+        if is_truthy(disable_ec2_metadata.to_lowercase().as_str()) {
+            cfg.disable_vm_metadata = true;
+        }
+    };
+    if let Some(disable_config_load) = m.remove(GCS_DISABLE_CONFIG_LOAD) {
+        if is_truthy(disable_config_load.to_lowercase().as_str()) {
+            cfg.disable_config_load = true;
+        }
+    };
+
     Ok(cfg)
 }
 
