@@ -18,7 +18,21 @@
 //! Iceberg Puffin implementation.
 
 #![deny(missing_docs)]
-// Temporarily allowing this while crate is under active development
-#![allow(dead_code)]
+
+mod blob;
+pub use blob::{APACHE_DATASKETCHES_THETA_V1, Blob, DELETION_VECTOR_V1};
 
 mod compression;
+pub use compression::CompressionCodec;
+
+mod metadata;
+pub use metadata::{BlobMetadata, CREATED_BY_PROPERTY, FileMetadata};
+
+mod reader;
+pub use reader::PuffinReader;
+
+mod writer;
+pub use writer::PuffinWriter;
+
+#[cfg(test)]
+mod test_utils;
