@@ -718,7 +718,7 @@ pub(super) mod _serde {
         },
         Struct {
             r#type: String,
-            fields: Cow<'a, Vec<NestedFieldRef>>,
+            fields: Cow<'a, [NestedFieldRef]>,
         },
         #[serde(rename_all = "kebab-case")]
         Map {
@@ -1177,7 +1177,7 @@ mod tests {
     }
 
     #[test]
-    fn test_primitive_type_compatitable() {
+    fn test_primitive_type_compatible() {
         let pairs = vec![
             (PrimitiveType::Boolean, PrimitiveLiteral::Boolean(true)),
             (PrimitiveType::Int, PrimitiveLiteral::Int(1)),
