@@ -545,7 +545,7 @@ mod tests {
                         upper_bounds: HashMap::new(),
                         key_metadata: Some(Vec::new()),
                         split_offsets: vec![4],
-                        equality_ids: Vec::new(),
+                        equality_ids: None,
                         sort_order_id: None,
                         partition_spec_id: 0,
                         first_row_id: None,
@@ -574,7 +574,7 @@ mod tests {
                         upper_bounds: HashMap::new(),
                         key_metadata: Some(Vec::new()),
                         split_offsets: vec![4],
-                        equality_ids: Vec::new(),
+                        equality_ids: None,
                         sort_order_id: None,
                         partition_spec_id: 0,
                         first_row_id: None,
@@ -603,7 +603,7 @@ mod tests {
                         upper_bounds: HashMap::new(),
                         key_metadata: Some(Vec::new()),
                         split_offsets: vec![4],
-                        equality_ids: Vec::new(),
+                        equality_ids: None,
                         sort_order_id: None,
                         partition_spec_id: 0,
                         first_row_id: None,
