@@ -168,6 +168,12 @@ impl Table {
         self
     }
 
+    /// Sets the [`Table`] metadata location and returns an updated instance.
+    pub(crate) fn with_metadata_location(mut self, metadata_location: String) -> Self {
+        self.metadata_location = Some(metadata_location);
+        self
+    }
+
     /// Returns a TableBuilder to build a table
     pub fn builder() -> TableBuilder {
         TableBuilder::new()
@@ -192,6 +198,17 @@ impl Table {
         self.metadata_location.as_deref()
     }
 
+    /// Returns current metadata location in a result.
+    pub fn metadata_location_result(&self) -> Result<&str> {
+        self.metadata_location.as_deref().ok_or(Error::new(
+            ErrorKind::DataInvalid,
+            format!(
+                "Metadata location does not exist for table: {}",
+                self.identifier
+            ),
+        ))
+    }
+
     /// Returns file io used in this table.
     pub fn file_io(&self) -> &FileIO {
         &self.file_io
@@ -271,14 +288,23 @@ impl StaticTable {
     }
     /// Creates a static table directly from metadata file and `FileIO`
     pub async fn from_metadata_file(
-        metadata_file_path: &str,
+        metadata_location: &str,
         table_ident: TableIdent,
         file_io: FileIO,
     ) -> Result<Self> {
-        let metadata_file = file_io.new_input(metadata_file_path)?;
+        let metadata_file = file_io.new_input(metadata_location)?;
         let metadata_file_content = metadata_file.read().await?;
-        let table_metadata = serde_json::from_slice::<TableMetadata>(&metadata_file_content)?;
-        Self::from_metadata(table_metadata, table_ident, file_io).await
+        let metadata = serde_json::from_slice::<TableMetadata>(&metadata_file_content)?;
+
+        let table = Table::builder()
+            .metadata(metadata)
+            .metadata_location(metadata_location)
+            .identifier(table_ident)
+            .file_io(file_io.clone())
+            .readonly(true)
+            .build();
+
+        Ok(Self(table?))
     }
 
     /// Create a TableScanBuilder for the static table.
@@ -356,6 +382,10 @@ mod tests {
         let table = static_table.into_table();
         assert!(table.readonly());
         assert_eq!(table.identifier.name(), "static_table");
+        assert_eq!(
+            table.metadata_location(),
+            Some(metadata_file_path).as_deref()
+        );
     }
 
     #[tokio::test]
