@@ -18,7 +18,7 @@
 use std::sync::Arc;
 
 use arrow_arith::arity::binary;
-use arrow_arith::temporal::{date_part, DatePart};
+use arrow_arith::temporal::{DatePart, date_part};
 use arrow_array::types::Date32Type;
 use arrow_array::{
     Array, ArrayRef, Date32Array, Int32Array, TimestampMicrosecondArray, TimestampNanosecondArray,
@@ -30,8 +30,10 @@ use super::TransformFunction;
 use crate::spec::{Datum, PrimitiveLiteral, PrimitiveType};
 use crate::{Error, ErrorKind, Result};
 
-/// Hour in one second.
-const HOUR_PER_SECOND: f64 = 1.0_f64 / 3600.0_f64;
+/// Microseconds in one hour.
+const MICROSECONDS_PER_HOUR: i64 = 3_600_000_000;
+/// Nanoseconds in one hour.
+const NANOSECONDS_PER_HOUR: i64 = 3_600_000_000_000;
 /// Year of unix epoch.
 const UNIX_EPOCH_YEAR: i32 = 1970;
 /// One second in micros.
@@ -100,7 +102,7 @@ impl TransformFunction for Year {
                         "Unsupported data type for year transform: {:?}",
                         input.data_type()
                     ),
-                ))
+                ));
             }
         };
         Ok(Some(Datum::int(val)))
@@ -198,7 +200,7 @@ impl TransformFunction for Month {
                         "Unsupported data type for month transform: {:?}",
                         input.data_type()
                     ),
-                ))
+                ));
             }
         };
         Ok(Some(Datum::int(val)))
@@ -295,7 +297,7 @@ impl TransformFunction for Day {
                         "Should not call internally for unsupported data type {:?}",
                         input.data_type()
                     ),
-                ))
+                ));
             }
         };
         Ok(Arc::new(res))
@@ -321,7 +323,7 @@ impl TransformFunction for Day {
                         "Unsupported data type for day transform: {:?}",
                         input.data_type()
                     ),
-                ))
+                ));
             }
         };
         Ok(Some(Datum::date(val)))
@@ -335,12 +337,12 @@ pub struct Hour;
 impl Hour {
     #[inline]
     fn hour_timestamp_micro(v: i64) -> i32 {
-        (v as f64 / 1000.0 / 1000.0 * HOUR_PER_SECOND) as i32
+        v.div_euclid(MICROSECONDS_PER_HOUR) as i32
     }
 
     #[inline]
     fn hour_timestamp_nano(v: i64) -> i32 {
-        (v as f64 / 1_000_000.0 / 1000.0 * HOUR_PER_SECOND) as i32
+        v.div_euclid(NANOSECONDS_PER_HOUR) as i32
     }
 }
 
@@ -384,7 +386,7 @@ impl TransformFunction for Hour {
                         "Unsupported data type for hour transform: {:?}",
                         input.data_type()
                     ),
-                ))
+                ));
             }
         };
         Ok(Some(Datum::int(val)))
@@ -398,6 +400,7 @@ mod test {
     use arrow_array::{ArrayRef, Date32Array, Int32Array, TimestampMicrosecondArray};
     use chrono::{NaiveDate, NaiveDateTime};
 
+    use crate::Result;
     use crate::expr::PredicateOperator;
     use crate::spec::PrimitiveType::{
         Binary, Date, Decimal, Fixed, Int, Long, String as StringType, Time, Timestamp,
@@ -407,7 +410,6 @@ mod test {
     use crate::spec::{Datum, NestedField, PrimitiveType, StructType, Transform, Type};
     use crate::transform::test::{TestProjectionFixture, TestTransformFixture};
     use crate::transform::{BoxedTransformFunction, TransformFunction};
-    use crate::Result;
 
     #[test]
     fn test_year_transform() {
@@ -447,12 +449,9 @@ mod test {
                 (Primitive(TimestampNs), Some(Primitive(Int))),
                 (Primitive(TimestamptzNs), Some(Primitive(Int))),
                 (
-                    Struct(StructType::new(vec![NestedField::optional(
-                        1,
-                        "a",
-                        Primitive(Timestamp),
-                    )
-                    .into()])),
+                    Struct(StructType::new(vec![
+                        NestedField::optional(1, "a", Primitive(Timestamp)).into(),
+                    ])),
                     None,
                 ),
             ],
@@ -499,12 +498,9 @@ mod test {
                 (Primitive(TimestampNs), Some(Primitive(Int))),
                 (Primitive(TimestamptzNs), Some(Primitive(Int))),
                 (
-                    Struct(StructType::new(vec![NestedField::optional(
-                        1,
-                        "a",
-                        Primitive(Timestamp),
-                    )
-                    .into()])),
+                    Struct(StructType::new(vec![
+                        NestedField::optional(1, "a", Primitive(Timestamp)).into(),
+                    ])),
                     None,
                 ),
             ],
@@ -551,12 +547,9 @@ mod test {
                 (Primitive(TimestampNs), Some(Primitive(Date))),
                 (Primitive(TimestamptzNs), Some(Primitive(Date))),
                 (
-                    Struct(StructType::new(vec![NestedField::optional(
-                        1,
-                        "a",
-                        Primitive(Timestamp),
-                    )
-                    .into()])),
+                    Struct(StructType::new(vec![
+                        NestedField::optional(1, "a", Primitive(Timestamp)).into(),
+                    ])),
                     None,
                 ),
             ],
@@ -603,12 +596,9 @@ mod test {
                 (Primitive(TimestampNs), Some(Primitive(Int))),
                 (Primitive(TimestamptzNs), Some(Primitive(Int))),
                 (
-                    Struct(StructType::new(vec![NestedField::optional(
-                        1,
-                        "a",
-                        Primitive(Timestamp),
-                    )
-                    .into()])),
+                    Struct(StructType::new(vec![
+                        NestedField::optional(1, "a", Primitive(Timestamp)).into(),
+                    ])),
                     None,
                 ),
             ],
@@ -2388,18 +2378,8 @@ mod test {
         transform: &BoxedTransformFunction,
         expect: Datum,
     ) {
-        let timestamp = Datum::timestamp_micros(
-            NaiveDateTime::parse_from_str(time, "%Y-%m-%d %H:%M:%S.%f")
-                .unwrap()
-                .and_utc()
-                .timestamp_micros(),
-        );
-        let timestamp_tz = Datum::timestamptz_micros(
-            NaiveDateTime::parse_from_str(time, "%Y-%m-%d %H:%M:%S.%f")
-                .unwrap()
-                .and_utc()
-                .timestamp_micros(),
-        );
+        let timestamp = Datum::timestamp_from_str(time).unwrap();
+        let timestamp_tz = Datum::timestamptz_from_str(time.to_owned() + " +00:00").unwrap();
         let res = transform.transform_literal(&timestamp).unwrap().unwrap();
         assert_eq!(res, expect);
         let res = transform.transform_literal(&timestamp_tz).unwrap().unwrap();
@@ -2430,20 +2410,8 @@ mod test {
         transform: &BoxedTransformFunction,
         expect: Datum,
     ) {
-        let timestamp_ns = Datum::timestamp_nanos(
-            NaiveDateTime::parse_from_str(time, "%Y-%m-%d %H:%M:%S.%f")
-                .unwrap()
-                .and_utc()
-                .timestamp_nanos_opt()
-                .unwrap(),
-        );
-        let timestamptz_ns = Datum::timestamptz_nanos(
-            NaiveDateTime::parse_from_str(time, "%Y-%m-%d %H:%M:%S.%f")
-                .unwrap()
-                .and_utc()
-                .timestamp_nanos_opt()
-                .unwrap(),
-        );
+        let timestamp_ns = Datum::timestamp_from_str(time).unwrap();
+        let timestamptz_ns = Datum::timestamptz_from_str(time.to_owned() + " +00:00").unwrap();
         let res = transform.transform_literal(&timestamp_ns).unwrap().unwrap();
         assert_eq!(res, expect);
         let res = transform
@@ -2483,7 +2451,7 @@ mod test {
             &year,
             Datum::int(1970 - super::UNIX_EPOCH_YEAR),
         );
-        test_timestamp_and_tz_transform("1969-01-01 00:00:00.00", &year, Datum::int(-1));
+        test_timestamp_and_tz_transform("1969-01-01T00:00:00.000000", &year, Datum::int(-1));
 
         // Test TimestampNanosecond
         test_timestamp_ns_and_tz_transform_using_i64(
@@ -2491,7 +2459,7 @@ mod test {
             &year,
             Datum::int(1970 - super::UNIX_EPOCH_YEAR),
         );
-        test_timestamp_ns_and_tz_transform("1969-01-01 00:00:00.00", &year, Datum::int(-1));
+        test_timestamp_ns_and_tz_transform("1969-01-01T00:00:00.000000", &year, Datum::int(-1));
     }
 
     #[test]
@@ -2582,10 +2550,10 @@ mod test {
             &month,
             Datum::int((1970 - super::UNIX_EPOCH_YEAR) * 12),
         );
-        test_timestamp_and_tz_transform("1969-12-01 23:00:00.00", &month, Datum::int(-1));
-        test_timestamp_and_tz_transform("2017-12-01 00:00:00.00", &month, Datum::int(575));
-        test_timestamp_and_tz_transform("1970-01-01 00:00:00.00", &month, Datum::int(0));
-        test_timestamp_and_tz_transform("1969-12-31 00:00:00.00", &month, Datum::int(-1));
+        test_timestamp_and_tz_transform("1969-12-01T23:00:00.000000", &month, Datum::int(-1));
+        test_timestamp_and_tz_transform("2017-12-01T00:00:00.000000", &month, Datum::int(575));
+        test_timestamp_and_tz_transform("1970-01-01T00:00:00.000000", &month, Datum::int(0));
+        test_timestamp_and_tz_transform("1969-12-31T00:00:00.000000", &month, Datum::int(-1));
 
         // Test TimestampNanosecond
         test_timestamp_ns_and_tz_transform_using_i64(
@@ -2593,10 +2561,10 @@ mod test {
             &month,
             Datum::int((1970 - super::UNIX_EPOCH_YEAR) * 12),
         );
-        test_timestamp_ns_and_tz_transform("1969-12-01 23:00:00.00", &month, Datum::int(-1));
-        test_timestamp_ns_and_tz_transform("2017-12-01 00:00:00.00", &month, Datum::int(575));
-        test_timestamp_ns_and_tz_transform("1970-01-01 00:00:00.00", &month, Datum::int(0));
-        test_timestamp_ns_and_tz_transform("1969-12-31 00:00:00.00", &month, Datum::int(-1));
+        test_timestamp_ns_and_tz_transform("1969-12-01T23:00:00.000000", &month, Datum::int(-1));
+        test_timestamp_ns_and_tz_transform("2017-12-01T00:00:00.000000", &month, Datum::int(575));
+        test_timestamp_ns_and_tz_transform("1970-01-01T00:00:00.000000", &month, Datum::int(0));
+        test_timestamp_ns_and_tz_transform("1969-12-31T00:00:00.000000", &month, Datum::int(-1));
     }
 
     #[test]
@@ -2687,12 +2655,12 @@ mod test {
         // Test TimestampMicrosecond
         test_timestamp_and_tz_transform_using_i64(1512151975038194, &day, Datum::date(17501));
         test_timestamp_and_tz_transform_using_i64(-115200000000, &day, Datum::date(-2));
-        test_timestamp_and_tz_transform("2017-12-01 10:30:42.123", &day, Datum::date(17501));
+        test_timestamp_and_tz_transform("2017-12-01T10:30:42.123000", &day, Datum::date(17501));
 
         // Test TimestampNanosecond
         test_timestamp_ns_and_tz_transform_using_i64(1512151975038194, &day, Datum::date(17));
         test_timestamp_ns_and_tz_transform_using_i64(-115200000000, &day, Datum::date(-1));
-        test_timestamp_ns_and_tz_transform("2017-12-01 10:30:42.123", &day, Datum::date(17501));
+        test_timestamp_ns_and_tz_transform("2017-12-01T10:30:42.123000", &day, Datum::date(17501));
     }
 
     #[test]
@@ -2758,12 +2726,23 @@ mod test {
     fn test_transform_hours_literal() {
         let hour = Box::new(super::Hour) as BoxedTransformFunction;
 
-        // Test TimestampMicrosecond
-        test_timestamp_and_tz_transform("2017-12-01 18:00:00.00", &hour, Datum::int(420042));
-        test_timestamp_and_tz_transform("1969-12-31 23:00:00.00", &hour, Datum::int(-1));
+        test_timestamp_and_tz_transform("2017-12-01T18:00:00.000000", &hour, Datum::int(420042));
+        test_timestamp_and_tz_transform("1970-01-01T22:01:01.000000", &hour, Datum::int(22));
+        test_timestamp_and_tz_transform("1969-12-31T23:00:00.000000", &hour, Datum::int(-1));
+        test_timestamp_and_tz_transform("1969-12-31T22:01:01.000000", &hour, Datum::int(-2));
+        test_timestamp_and_tz_transform("0022-05-01T22:01:01.000000", &hour, Datum::int(-17072906));
 
         // Test TimestampNanosecond
-        test_timestamp_ns_and_tz_transform("2017-12-01 18:00:00.00", &hour, Datum::int(420042));
-        test_timestamp_ns_and_tz_transform("1969-12-31 23:00:00.00", &hour, Datum::int(-1));
+        test_timestamp_ns_and_tz_transform(
+            "2017-12-01T18:00:00.0000000000",
+            &hour,
+            Datum::int(420042),
+        );
+        test_timestamp_ns_and_tz_transform("1969-12-31T23:00:00.0000000000", &hour, Datum::int(-1));
+        test_timestamp_ns_and_tz_transform(
+            "1900-05-01T22:01:01.0000000000",
+            &hour,
+            Datum::int(-610706),
+        );
     }
 }
