@@ -18,7 +18,7 @@
 use std::collections::HashMap;
 
 use iceberg::io::{S3_ACCESS_KEY_ID, S3_ENDPOINT, S3_REGION, S3_SECRET_ACCESS_KEY};
-use iceberg_catalog_rest::RestCatalogConfig;
+use iceberg_catalog_rest::REST_CATALOG_PROP_URI;
 use iceberg_test_utils::docker::DockerCompose;
 use iceberg_test_utils::{normalize_test_name, set_up};
 
@@ -26,7 +26,7 @@ const REST_CATALOG_PORT: u16 = 8181;
 
 pub struct TestFixture {
     pub _docker_compose: DockerCompose,
-    pub catalog_config: RestCatalogConfig,
+    pub catalog_config: HashMap<String, String>,
 }
 
 pub fn set_test_fixture(func: &str) -> TestFixture {
@@ -43,18 +43,19 @@ pub fn set_test_fixture(func: &str) -> TestFixture {
     let rest_catalog_ip = docker_compose.get_container_ip("rest");
     let minio_ip = docker_compose.get_container_ip("minio");
 
-    let catalog_config = RestCatalogConfig::builder()
-        .uri(format!("http://{}:{}", rest_catalog_ip, REST_CATALOG_PORT))
-        .props(HashMap::from([
-            (
-                S3_ENDPOINT.to_string(),
-                format!("http://{}:{}", minio_ip, 9000),
-            ),
-            (S3_ACCESS_KEY_ID.to_string(), "admin".to_string()),
-            (S3_SECRET_ACCESS_KEY.to_string(), "password".to_string()),
-            (S3_REGION.to_string(), "us-east-1".to_string()),
-        ]))
-        .build();
+    let catalog_config = HashMap::from([
+        (
+            REST_CATALOG_PROP_URI.to_string(),
+            format!("http://{}:{}", rest_catalog_ip, REST_CATALOG_PORT),
+        ),
+        (
+            S3_ENDPOINT.to_string(),
+            format!("http://{}:{}", minio_ip, 9000),
+        ),
+        (S3_ACCESS_KEY_ID.to_string(), "admin".to_string()),
+        (S3_SECRET_ACCESS_KEY.to_string(), "password".to_string()),
+        (S3_REGION.to_string(), "us-east-1".to_string()),
+    ]);
 
     TestFixture {
         _docker_compose: docker_compose,
