@@ -21,7 +21,9 @@ use std::sync::Arc;
 
 use arrow_array::{ArrayRef, BooleanArray, Int32Array, RecordBatch, StringArray};
 use futures::TryStreamExt;
-use iceberg::spec::{Literal, PrimitiveLiteral, Struct, Transform, UnboundPartitionSpec};
+use iceberg::spec::{
+    Literal, PartitionKey, PrimitiveLiteral, Struct, Transform, UnboundPartitionSpec,
+};
 use iceberg::table::Table;
 use iceberg::transaction::{ApplyTransactionAction, Transaction};
 use iceberg::writer::base_writer::data_file_writer::DataFileWriterBuilder;
@@ -30,8 +32,8 @@ use iceberg::writer::file_writer::location_generator::{
     DefaultFileNameGenerator, DefaultLocationGenerator,
 };
 use iceberg::writer::{IcebergWriter, IcebergWriterBuilder};
-use iceberg::{Catalog, TableCreation};
-use iceberg_catalog_rest::RestCatalog;
+use iceberg::{Catalog, CatalogBuilder, TableCreation};
+use iceberg_catalog_rest::RestCatalogBuilder;
 use parquet::file::properties::WriterProperties;
 
 use crate::get_shared_containers;
@@ -40,7 +42,10 @@ use crate::shared_tests::{random_ns, test_schema};
 #[tokio::test]
 async fn test_append_partition_data_file() {
     let fixture = get_shared_containers();
-    let rest_catalog = RestCatalog::new(fixture.catalog_config.clone());
+    let rest_catalog = RestCatalogBuilder::default()
+        .load("rest", fixture.catalog_config.clone())
+        .await
+        .unwrap();
     let ns = random_ns().await;
     let schema = test_schema();
 
@@ -56,7 +61,7 @@ async fn test_append_partition_data_file() {
     let table_creation = TableCreation::builder()
         .name("t1".to_string())
         .schema(schema.clone())
-        .partition_spec(partition_spec)
+        .partition_spec(partition_spec.clone())
         .build();
 
     let table = rest_catalog
@@ -75,6 +80,11 @@ async fn test_append_partition_data_file() {
     );
 
     let first_partition_id_value = 100;
+    let partition_key = PartitionKey::new(
+        partition_spec.clone(),
+        table.metadata().current_schema().clone(),
+        Struct::from_iter(vec![Some(Literal::int(first_partition_id_value))]),
+    );
 
     let location_generator = DefaultLocationGenerator::new(table.metadata().clone()).unwrap();
     let file_name_generator = DefaultFileNameGenerator::new(
@@ -86,6 +96,7 @@ async fn test_append_partition_data_file() {
     let parquet_writer_builder = ParquetWriterBuilder::new(
         WriterProperties::default(),
         table.metadata().current_schema().clone(),
+        Some(partition_key),
         table.file_io().clone(),
         location_generator.clone(),
         file_name_generator.clone(),
