@@ -28,8 +28,8 @@ use iceberg::writer::file_writer::location_generator::{
     DefaultFileNameGenerator, DefaultLocationGenerator,
 };
 use iceberg::writer::{IcebergWriter, IcebergWriterBuilder};
-use iceberg::{Catalog, TableCreation};
-use iceberg_catalog_rest::RestCatalog;
+use iceberg::{Catalog, CatalogBuilder, TableCreation};
+use iceberg_catalog_rest::RestCatalogBuilder;
 use parquet::file::properties::WriterProperties;
 
 use crate::get_shared_containers;
@@ -38,7 +38,10 @@ use crate::shared_tests::{random_ns, test_schema};
 #[tokio::test]
 async fn test_append_data_file_conflict() {
     let fixture = get_shared_containers();
-    let rest_catalog = RestCatalog::new(fixture.catalog_config.clone());
+    let rest_catalog = RestCatalogBuilder::default()
+        .load("rest", fixture.catalog_config.clone())
+        .await
+        .unwrap();
     let ns = random_ns().await;
     let schema = test_schema();
 
@@ -70,6 +73,7 @@ async fn test_append_data_file_conflict() {
     let parquet_writer_builder = ParquetWriterBuilder::new(
         WriterProperties::default(),
         table.metadata().current_schema().clone(),
+        None,
         table.file_io().clone(),
         location_generator.clone(),
         file_name_generator.clone(),
