@@ -15,6 +15,12 @@
 // specific language governing permissions and limitations
 // under the License.
 
+pub(crate) mod commit;
 pub(crate) mod expr_to_predicate;
 pub(crate) mod metadata_scan;
 pub(crate) mod scan;
+pub(crate) mod write;
+
+pub(crate) const DATA_FILES_COL_NAME: &str = "data_files";
+
+pub use scan::IcebergTableScan;
